@@ -3,16 +3,23 @@
 # License: GNU Affero General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
+from datetime import datetime, timezone
 import random
 import re
 import string
+import textwrap
 
 from django.utils.html import escape
 
 from hypothesis import given
 
 from swh.model.hashutil import hash_to_bytes
-from swh.model.model import Snapshot
+from swh.model.identifiers import CONTENT, DIRECTORY, RELEASE, REVISION, SNAPSHOT
+from swh.model.model import (
+    Snapshot,
+    SnapshotBranch,
+    TargetType,
+)
 from swh.web.browse.snapshot_context import process_snapshot_branches
 from swh.web.common.exc import NotFoundExc
 from swh.web.common.identifiers import get_swh_persistent_id
@@ -22,7 +29,6 @@ from swh.web.common.utils import (
     format_utc_iso_date,
     parse_timestamp,
 )
-from swh.web.config import get_config
 from swh.web.tests.data import get_content, random_sha1
 from swh.web.tests.django_asserts import assert_contains, assert_template_used
 from swh.web.tests.strategies import (
@@ -34,18 +40,19 @@ from swh.web.tests.strategies import (
     revisions,
     origin_with_releases,
     release as existing_release,
+    unknown_revision,
 )
 
 
 @given(origin_with_multiple_visits())
 def test_origin_visits_browse(client, archive_data, origin):
-    url = reverse("browse-origin-visits", url_args={"origin_url": origin["url"]})
+    url = reverse("browse-origin-visits", query_params={"origin_url": origin["url"]})
     resp = client.get(url)
 
     assert resp.status_code == 200
     assert_template_used(resp, "browse/origin-visits.html")
 
-    url = reverse("browse-origin-visits", url_args={"origin_url": origin["url"]})
+    url = reverse("browse-origin-visits", query_params={"origin_url": origin["url"]})
     resp = client.get(url)
 
     assert resp.status_code == 200
@@ -57,10 +64,12 @@ def test_origin_visits_browse(client, archive_data, origin):
         vdate = format_utc_iso_date(v["date"], "%Y-%m-%dT%H:%M:%SZ")
         browse_dir_url = reverse(
             "browse-origin-directory",
-            url_args={"origin_url": origin["url"], "timestamp": vdate},
+            query_params={"origin_url": origin["url"], "timestamp": vdate},
         )
         assert_contains(resp, browse_dir_url)
 
+    _check_origin_view_title(resp, origin["url"], "visits")
+
 
 @given(origin_with_multiple_visits())
 def test_origin_content_view(client, archive_data, origin):
@@ -86,8 +95,9 @@ def test_origin_content_view(client, archive_data, origin):
 
     _origin_content_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        origin_visits[-1],
         tdata["branches"],
         tdata["releases"],
         tdata["root_dir_sha1"],
@@ -96,8 +106,9 @@ def test_origin_content_view(client, archive_data, origin):
 
     _origin_content_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        origin_visits[-1],
         tdata["branches"],
         tdata["releases"],
         tdata["root_dir_sha1"],
@@ -110,8 +121,9 @@ def test_origin_content_view(client, archive_data, origin):
 
     _origin_content_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        origin_visits[-1],
         tdata["branches"],
         tdata["releases"],
         tdata["root_dir_sha1"],
@@ -123,8 +135,9 @@ def test_origin_content_view(client, archive_data, origin):
 
     _origin_content_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        origin_visits[0],
         tdata["branches"],
         tdata["releases"],
         tdata["root_dir_sha1"],
@@ -148,13 +161,21 @@ def test_origin_root_directory_view(client, archive_data, origin):
     visit_unix_ts = int(visit_unix_ts)
 
     _origin_directory_view_test_helper(
-        client, origin, origin_visits, branches, releases, root_dir_sha1, dir_content
+        client,
+        archive_data,
+        origin,
+        visit,
+        branches,
+        releases,
+        root_dir_sha1,
+        dir_content,
     )
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -164,8 +185,9 @@ def test_origin_root_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -175,8 +197,9 @@ def test_origin_root_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -188,13 +211,21 @@ def test_origin_root_directory_view(client, archive_data, origin):
     del origin["type"]
 
     _origin_directory_view_test_helper(
-        client, origin, origin_visits, branches, releases, root_dir_sha1, dir_content
+        client,
+        archive_data,
+        origin,
+        visit,
+        branches,
+        releases,
+        root_dir_sha1,
+        dir_content,
     )
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -204,8 +235,9 @@ def test_origin_root_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -215,8 +247,9 @@ def test_origin_root_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -250,8 +283,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -261,8 +295,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -273,8 +308,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -285,8 +321,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -300,8 +337,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -311,8 +349,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -323,8 +362,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -335,8 +375,9 @@ def test_origin_sub_directory_view(client, archive_data, origin):
 
     _origin_directory_view_test_helper(
         client,
+        archive_data,
         origin,
-        origin_visits,
+        visit,
         branches,
         releases,
         root_dir_sha1,
@@ -404,7 +445,9 @@ def test_origin_snapshot_null_branch(
         new_origin["url"], visit.visit, status="partial", snapshot=snp_dict["id"]
     )
 
-    url = reverse("browse-origin-directory", url_args={"origin_url": new_origin["url"]})
+    url = reverse(
+        "browse-origin-directory", query_params={"origin_url": new_origin["url"]}
+    )
     rv = client.get(url)
     assert rv.status_code == 200
 
@@ -434,8 +477,7 @@ def test_origin_snapshot_invalid_branch(
 
     url = reverse(
         "browse-origin-directory",
-        url_args={"origin_url": new_origin["url"]},
-        query_params={"branch": "invalid_branch"},
+        query_params={"origin_url": new_origin["url"], "branch": "invalid_branch"},
     )
     rv = client.get(url)
     assert rv.status_code == 404
@@ -443,7 +485,7 @@ def test_origin_snapshot_invalid_branch(
 
 @given(new_origin())
 def test_browse_visits_origin_not_found(client, new_origin):
-    url = reverse("browse-origin-visits", url_args={"origin_url": new_origin.url})
+    url = reverse("browse-origin-visits", query_params={"origin_url": new_origin.url})
     resp = client.get(url)
     assert resp.status_code == 404
     assert_template_used(resp, "error.html")
@@ -458,7 +500,7 @@ def test_browse_origin_directory_no_visit(client, mocker, origin):
         "swh.web.common.origin_visits.get_origin_visits"
     )
     mock_get_origin_visits.return_value = []
-    url = reverse("browse-origin-directory", url_args={"origin_url": origin["url"]})
+    url = reverse("browse-origin-directory", query_params={"origin_url": origin["url"]})
     resp = client.get(url)
     assert resp.status_code == 404
     assert_template_used(resp, "error.html")
@@ -475,8 +517,7 @@ def test_browse_origin_directory_unknown_visit(client, mocker, origin):
 
     url = reverse(
         "browse-origin-directory",
-        url_args={"origin_url": origin["url"]},
-        query_params={"visit_id": 2},
+        query_params={"origin_url": origin["url"], "visit_id": 2},
     )
     resp = client.get(url)
     assert resp.status_code == 404
@@ -489,7 +530,7 @@ def test_browse_origin_directory_unknown_visit(client, mocker, origin):
 def test_browse_origin_directory_not_found(client, origin):
     url = reverse(
         "browse-origin-directory",
-        url_args={"origin_url": origin["url"], "path": "/invalid/dir/path/"},
+        query_params={"origin_url": origin["url"], "path": "/invalid/dir/path/"},
     )
     resp = client.get(url)
     assert resp.status_code == 404
@@ -504,7 +545,8 @@ def test_browse_origin_content_no_visit(client, mocker, origin):
     )
     mock_get_origin_visits.return_value = []
     url = reverse(
-        "browse-origin-content", url_args={"origin_url": origin["url"], "path": "foo"}
+        "browse-origin-content",
+        query_params={"origin_url": origin["url"], "path": "foo"},
     )
     resp = client.get(url)
     assert resp.status_code == 404
@@ -522,8 +564,7 @@ def test_browse_origin_content_unknown_visit(client, mocker, origin):
 
     url = reverse(
         "browse-origin-content",
-        url_args={"origin_url": origin["url"], "path": "foo"},
-        query_params={"visit_id": 2},
+        query_params={"origin_url": origin["url"], "path": "foo", "visit_id": 2},
     )
     resp = client.get(url)
     assert resp.status_code == 404
@@ -533,7 +574,7 @@ def test_browse_origin_content_unknown_visit(client, mocker, origin):
 
 
 @given(origin())
-def test_browse_origin_content_empty_snapshot(client, mocker, origin):
+def test_browse_origin_content_directory_empty_snapshot(client, mocker, origin):
     mock_snapshot_service = mocker.patch("swh.web.browse.snapshot_context.service")
     mock_get_origin_visit_snapshot = mocker.patch(
         "swh.web.browse.snapshot_context.get_origin_visit_snapshot"
@@ -544,23 +585,27 @@ def test_browse_origin_content_empty_snapshot(client, mocker, origin):
         "revision": 0,
         "release": 0,
     }
-    url = reverse(
-        "browse-origin-content", url_args={"origin_url": origin["url"], "path": "baz"}
-    )
-    resp = client.get(url)
-    assert resp.status_code == 200
-    assert_template_used(resp, "browse/content.html")
-    assert re.search("snapshot.*is empty", resp.content.decode("utf-8"))
-    assert mock_get_origin_visit_snapshot.called
-    assert mock_snapshot_service.lookup_origin.called
-    assert mock_snapshot_service.lookup_snapshot_sizes.called
+
+    for browse_context in ("content", "directory"):
+
+        url = reverse(
+            f"browse-origin-{browse_context}",
+            query_params={"origin_url": origin["url"], "path": "baz"},
+        )
+        resp = client.get(url)
+        assert resp.status_code == 200
+        assert_template_used(resp, f"browse/{browse_context}.html")
+        assert re.search("snapshot.*is empty", resp.content.decode("utf-8"))
+        assert mock_get_origin_visit_snapshot.called
+        assert mock_snapshot_service.lookup_origin.called
+        assert mock_snapshot_service.lookup_snapshot_sizes.called
 
 
 @given(origin())
 def test_browse_origin_content_not_found(client, origin):
     url = reverse(
         "browse-origin-content",
-        url_args={"origin_url": origin["url"], "path": "/invalid/file/path"},
+        query_params={"origin_url": origin["url"], "path": "/invalid/file/path"},
     )
     resp = client.get(url)
     assert resp.status_code == 404
@@ -574,7 +619,7 @@ def test_browse_directory_snapshot_not_found(client, mocker, origin):
         "swh.web.browse.snapshot_context.get_snapshot_context"
     )
     mock_get_snapshot_context.side_effect = NotFoundExc("Snapshot not found")
-    url = reverse("browse-origin-directory", url_args={"origin_url": origin["url"]})
+    url = reverse("browse-origin-directory", query_params={"origin_url": origin["url"]})
     resp = client.get(url)
     assert resp.status_code == 404
     assert_template_used(resp, "error.html")
@@ -594,7 +639,7 @@ def test_origin_empty_snapshot(client, mocker, origin):
         "release": 0,
     }
     mock_service.lookup_origin.return_value = origin
-    url = reverse("browse-origin-directory", url_args={"origin_url": "bar"})
+    url = reverse("browse-origin-directory", query_params={"origin_url": origin["url"]})
     resp = client.get(url)
     assert resp.status_code == 200
     assert_template_used(resp, "browse/directory.html")
@@ -607,28 +652,35 @@ def test_origin_empty_snapshot(client, mocker, origin):
 
 @given(origin_with_releases())
 def test_origin_release_browse(client, archive_data, origin):
-    # for swh.web.browse.snapshot_context.get_snapshot_content to only return one branch
-    config = get_config()
-    snapshot_max_size = int(config["snapshot_content_max_size"])
-    config["snapshot_content_max_size"] = 1
-    try:
-        snapshot = archive_data.snapshot_get_latest(origin["url"])
-        release = [
-            b for b in snapshot["branches"].values() if b["target_type"] == "release"
-        ][-1]
-        release_data = archive_data.release_get(release["target"])
-        url = reverse(
-            "browse-origin-directory",
-            url_args={"origin_url": origin["url"]},
-            query_params={"release": release_data["name"]},
-        )
+    snapshot = archive_data.snapshot_get_latest(origin["url"])
+    release = [
+        b for b in snapshot["branches"].values() if b["target_type"] == "release"
+    ][-1]
+    release_data = archive_data.release_get(release["target"])
+    revision_data = archive_data.revision_get(release_data["target"])
+    url = reverse(
+        "browse-origin-directory",
+        query_params={"origin_url": origin["url"], "release": release_data["name"]},
+    )
 
-        resp = client.get(url)
-        assert resp.status_code == 200
-        assert_contains(resp, release_data["name"])
-        assert_contains(resp, release["target"])
-    finally:
-        config["snapshot_content_max_size"] = snapshot_max_size
+    resp = client.get(url)
+    assert resp.status_code == 200
+    assert_contains(resp, release_data["name"])
+    assert_contains(resp, release["target"])
+
+    swhid_context = {
+        "origin": origin["url"],
+        "visit": get_swh_persistent_id(SNAPSHOT, snapshot["id"]),
+        "anchor": get_swh_persistent_id(RELEASE, release_data["id"]),
+        "path": f"/",
+    }
+
+    swh_dir_id = get_swh_persistent_id(
+        DIRECTORY, revision_data["directory"], metadata=swhid_context
+    )
+    swh_dir_id_url = reverse("browse-swh-id", url_args={"swh_id": swh_dir_id})
+    assert_contains(resp, swh_dir_id)
+    assert_contains(resp, swh_dir_id_url)
 
 
 @given(origin_with_releases())
@@ -637,8 +689,7 @@ def test_origin_release_browse_not_found(client, origin):
     invalid_release_name = "swh-foo-bar"
     url = reverse(
         "browse-origin-directory",
-        url_args={"origin_url": origin["url"]},
-        query_params={"release": invalid_release_name},
+        query_params={"origin_url": origin["url"], "release": invalid_release_name},
     )
 
     resp = client.get(url)
@@ -648,10 +699,74 @@ def test_origin_release_browse_not_found(client, origin):
     )
 
 
+@given(new_origin(), unknown_revision())
+def test_origin_browse_directory_branch_with_non_resolvable_revision(
+    client, archive_data, new_origin, unknown_revision
+):
+    branch_name = "master"
+    snapshot = Snapshot(
+        branches={
+            branch_name.encode(): SnapshotBranch(
+                target=hash_to_bytes(unknown_revision), target_type=TargetType.REVISION,
+            )
+        }
+    )
+    new_origin = archive_data.origin_add([new_origin])[0]
+    archive_data.snapshot_add([snapshot])
+    visit = archive_data.origin_visit_add(
+        new_origin["url"], datetime.now(tz=timezone.utc), type="git"
+    )
+    archive_data.origin_visit_update(
+        new_origin["url"], visit.visit, status="full", snapshot=snapshot.id
+    )
+
+    url = reverse(
+        "browse-origin-directory",
+        query_params={"origin_url": new_origin["url"], "branch": branch_name},
+    )
+
+    resp = client.get(url)
+
+    assert resp.status_code == 200
+    assert_contains(
+        resp, f"Revision {unknown_revision } could not be found in the archive."
+    )
+
+
+@given(origin())
+def test_origin_content_no_path(client, origin):
+    url = reverse("browse-origin-content", query_params={"origin_url": origin["url"]})
+
+    resp = client.get(url)
+
+    assert resp.status_code == 400
+    assert_contains(
+        resp, "The path of a content must be given as query parameter.", status_code=400
+    )
+
+
+def test_origin_views_no_url_query_parameter(client):
+    for browse_context in (
+        "content",
+        "directory",
+        "log",
+        "branches",
+        "releases",
+        "visits",
+    ):
+        url = reverse(f"browse-origin-{browse_context}")
+        resp = client.get(url)
+        assert resp.status_code == 400
+        assert_contains(
+            resp, "An origin URL must be provided as query parameter.", status_code=400
+        )
+
+
 def _origin_content_view_test_helper(
     client,
+    archive_data,
     origin_info,
-    origin_visits,
+    origin_visit,
     origin_branches,
     origin_releases,
     root_dir_sha1,
@@ -661,20 +776,18 @@ def _origin_content_view_test_helper(
 ):
     content_path = "/".join(content["path"].split("/")[1:])
 
-    url_args = {"origin_url": origin_info["url"], "path": content_path}
-
     if not visit_id:
-        visit_id = origin_visits[-1]["visit"]
+        visit_id = origin_visit["visit"]
 
-    query_params = {}
+    query_params = {"origin_url": origin_info["url"], "path": content_path}
 
     if timestamp:
-        url_args["timestamp"] = timestamp
+        query_params["timestamp"] = timestamp
 
     if visit_id:
         query_params["visit_id"] = visit_id
 
-    url = reverse("browse-origin-content", url_args=url_args, query_params=query_params)
+    url = reverse("browse-origin-content", query_params=query_params)
 
     resp = client.get(url)
 
@@ -693,26 +806,22 @@ def _origin_content_view_test_helper(
 
     path_info = gen_path_info(path)
 
-    del url_args["path"]
+    del query_params["path"]
 
     if timestamp:
-        url_args["timestamp"] = format_utc_iso_date(
-            parse_timestamp(timestamp).isoformat(), "%Y-%m-%dT%H:%M:%S"
+        query_params["timestamp"] = format_utc_iso_date(
+            parse_timestamp(timestamp).isoformat(), "%Y-%m-%dT%H:%M:%SZ"
         )
 
-    root_dir_url = reverse(
-        "browse-origin-directory", url_args=url_args, query_params=query_params
-    )
+    root_dir_url = reverse("browse-origin-directory", query_params=query_params)
 
     assert_contains(resp, '<li class="swh-path">', count=len(path_info) + 1)
 
     assert_contains(resp, '<a href="%s">%s</a>' % (root_dir_url, root_dir_sha1[:7]))
 
     for p in path_info:
-        url_args["path"] = p["path"]
-        dir_url = reverse(
-            "browse-origin-directory", url_args=url_args, query_params=query_params
-        )
+        query_params["path"] = p["path"]
+        dir_url = reverse("browse-origin-directory", query_params=query_params)
         assert_contains(resp, '<a href="%s">%s</a>' % (dir_url, p["name"]))
 
     assert_contains(resp, "<li>%s</li>" % filename)
@@ -726,35 +835,33 @@ def _origin_content_view_test_helper(
     )
     assert_contains(resp, url_raw)
 
-    if "args" in url_args:
-        del url_args["path"]
+    if "path" in query_params:
+        del query_params["path"]
 
-    origin_branches_url = reverse(
-        "browse-origin-branches", url_args=url_args, query_params=query_params
-    )
+    origin_branches_url = reverse("browse-origin-branches", query_params=query_params)
 
     assert_contains(
         resp,
-        '<a href="%s">Branches (%s)</a>' % (origin_branches_url, len(origin_branches)),
+        '<a href="%s">Branches (%s)</a>'
+        % (escape(origin_branches_url), len(origin_branches)),
     )
 
-    origin_releases_url = reverse(
-        "browse-origin-releases", url_args=url_args, query_params=query_params
-    )
+    origin_releases_url = reverse("browse-origin-releases", query_params=query_params)
 
     assert_contains(
         resp,
-        '<a href="%s">Releases (%s)</a>' % (origin_releases_url, len(origin_releases)),
+        '<a href="%s">Releases (%s)</a>'
+        % (escape(origin_releases_url), len(origin_releases)),
     )
 
     assert_contains(resp, '<li class="swh-branch">', count=len(origin_branches))
 
-    url_args["path"] = content_path
+    query_params["path"] = content_path
 
     for branch in origin_branches:
-        query_params["branch"] = branch["name"]
         root_dir_branch_url = reverse(
-            "browse-origin-content", url_args=url_args, query_params=query_params
+            "browse-origin-content",
+            query_params={"branch": branch["name"], **query_params},
         )
 
         assert_contains(resp, '<a href="%s">' % root_dir_branch_url)
@@ -763,31 +870,46 @@ def _origin_content_view_test_helper(
 
     query_params["branch"] = None
     for release in origin_releases:
-        query_params["release"] = release["name"]
         root_dir_release_url = reverse(
-            "browse-origin-content", url_args=url_args, query_params=query_params
+            "browse-origin-content",
+            query_params={"release": release["name"], **query_params},
         )
 
         assert_contains(resp, '<a href="%s">' % root_dir_release_url)
 
-    url = reverse("browse-origin-content", url_args=url_args, query_params=query_params)
+    url = reverse("browse-origin-content", query_params=query_params)
 
     resp = client.get(url)
     assert resp.status_code == 200
     assert_template_used(resp, "browse/content.html")
 
-    swh_cnt_id = get_swh_persistent_id("content", content["sha1_git"])
+    snapshot = archive_data.snapshot_get(origin_visit["snapshot"])
+    head_rev_id = archive_data.snapshot_get_head(snapshot)
+
+    swhid_context = {
+        "origin": origin_info["url"],
+        "visit": get_swh_persistent_id(SNAPSHOT, snapshot["id"]),
+        "anchor": get_swh_persistent_id(REVISION, head_rev_id),
+        "path": f"/{content_path}",
+    }
+
+    swh_cnt_id = get_swh_persistent_id(
+        CONTENT, content["sha1_git"], metadata=swhid_context
+    )
     swh_cnt_id_url = reverse("browse-swh-id", url_args={"swh_id": swh_cnt_id})
     assert_contains(resp, swh_cnt_id)
     assert_contains(resp, swh_cnt_id_url)
 
     assert_contains(resp, "swh-take-new-snapshot")
 
+    _check_origin_view_title(resp, origin_info["url"], "content")
+
 
 def _origin_directory_view_test_helper(
     client,
+    archive_data,
     origin_info,
-    origin_visits,
+    origin_visit,
     origin_branches,
     origin_releases,
     root_directory_sha1,
@@ -800,23 +922,19 @@ def _origin_directory_view_test_helper(
     files = [e for e in directory_entries if e["type"] == "file"]
 
     if not visit_id:
-        visit_id = origin_visits[-1]["visit"]
+        visit_id = origin_visit["visit"]
 
-    url_args = {"origin_url": origin_info["url"]}
-
-    query_params = {}
+    query_params = {"origin_url": origin_info["url"]}
 
     if timestamp:
-        url_args["timestamp"] = timestamp
+        query_params["timestamp"] = timestamp
     else:
         query_params["visit_id"] = visit_id
 
     if path:
-        url_args["path"] = path
+        query_params["path"] = path
 
-    url = reverse(
-        "browse-origin-directory", url_args=url_args, query_params=query_params
-    )
+    url = reverse("browse-origin-directory", query_params=query_params)
 
     resp = client.get(url)
 
@@ -830,8 +948,8 @@ def _origin_directory_view_test_helper(
     assert_contains(resp, '<td class="swh-content">', count=len(files))
 
     if timestamp:
-        url_args["timestamp"] = format_utc_iso_date(
-            parse_timestamp(timestamp).isoformat(), "%Y-%m-%dT%H:%M:%S"
+        query_params["timestamp"] = format_utc_iso_date(
+            parse_timestamp(timestamp).isoformat(), "%Y-%m-%dT%H:%M:%SZ"
         )
 
     for d in dirs:
@@ -841,32 +959,22 @@ def _origin_directory_view_test_helper(
             dir_path = d["name"]
             if path:
                 dir_path = "%s/%s" % (path, d["name"])
-            dir_url_args = dict(url_args)
-            dir_url_args["path"] = dir_path
-            dir_url = reverse(
-                "browse-origin-directory",
-                url_args=dir_url_args,
-                query_params=query_params,
-            )
+            query_params["path"] = dir_path
+            dir_url = reverse("browse-origin-directory", query_params=query_params,)
         assert_contains(resp, dir_url)
 
     for f in files:
         file_path = f["name"]
         if path:
             file_path = "%s/%s" % (path, f["name"])
-        file_url_args = dict(url_args)
-        file_url_args["path"] = file_path
-        file_url = reverse(
-            "browse-origin-content", url_args=file_url_args, query_params=query_params
-        )
+        query_params["path"] = file_path
+        file_url = reverse("browse-origin-content", query_params=query_params)
         assert_contains(resp, file_url)
 
-    if "path" in url_args:
-        del url_args["path"]
+    if "path" in query_params:
+        del query_params["path"]
 
-    root_dir_branch_url = reverse(
-        "browse-origin-directory", url_args=url_args, query_params=query_params
-    )
+    root_dir_branch_url = reverse("browse-origin-directory", query_params=query_params)
 
     nb_bc_paths = 1
     if path:
@@ -877,34 +985,33 @@ def _origin_directory_view_test_helper(
         resp, '<a href="%s">%s</a>' % (root_dir_branch_url, root_directory_sha1[:7])
     )
 
-    origin_branches_url = reverse(
-        "browse-origin-branches", url_args=url_args, query_params=query_params
-    )
+    origin_branches_url = reverse("browse-origin-branches", query_params=query_params)
 
     assert_contains(
         resp,
-        '<a href="%s">Branches (%s)</a>' % (origin_branches_url, len(origin_branches)),
+        '<a href="%s">Branches (%s)</a>'
+        % (escape(origin_branches_url), len(origin_branches)),
     )
 
-    origin_releases_url = reverse(
-        "browse-origin-releases", url_args=url_args, query_params=query_params
-    )
+    origin_releases_url = reverse("browse-origin-releases", query_params=query_params)
 
     nb_releases = len(origin_releases)
     if nb_releases > 0:
         assert_contains(
-            resp, '<a href="%s">Releases (%s)</a>' % (origin_releases_url, nb_releases)
+            resp,
+            '<a href="%s">Releases (%s)</a>'
+            % (escape(origin_releases_url), nb_releases),
         )
 
     if path:
-        url_args["path"] = path
+        query_params["path"] = path
 
     assert_contains(resp, '<li class="swh-branch">', count=len(origin_branches))
 
     for branch in origin_branches:
         query_params["branch"] = branch["name"]
         root_dir_branch_url = reverse(
-            "browse-origin-directory", url_args=url_args, query_params=query_params
+            "browse-origin-directory", query_params=query_params
         )
 
         assert_contains(resp, '<a href="%s">' % root_dir_branch_url)
@@ -915,7 +1022,7 @@ def _origin_directory_view_test_helper(
     for release in origin_releases:
         query_params["release"] = release["name"]
         root_dir_release_url = reverse(
-            "browse-origin-directory", url_args=url_args, query_params=query_params
+            "browse-origin-directory", query_params=query_params
         )
 
         assert_contains(resp, '<a href="%s">' % root_dir_release_url)
@@ -923,18 +1030,32 @@ def _origin_directory_view_test_helper(
     assert_contains(resp, "vault-cook-directory")
     assert_contains(resp, "vault-cook-revision")
 
-    swh_dir_id = get_swh_persistent_id("directory", directory_entries[0]["dir_id"])
+    snapshot = archive_data.snapshot_get(origin_visit["snapshot"])
+    head_rev_id = archive_data.snapshot_get_head(snapshot)
+
+    swhid_context = {
+        "origin": origin_info["url"],
+        "visit": get_swh_persistent_id(SNAPSHOT, snapshot["id"]),
+        "anchor": get_swh_persistent_id(REVISION, head_rev_id),
+        "path": f"/{path}" if path else "/",
+    }
+
+    swh_dir_id = get_swh_persistent_id(
+        DIRECTORY, directory_entries[0]["dir_id"], metadata=swhid_context
+    )
     swh_dir_id_url = reverse("browse-swh-id", url_args={"swh_id": swh_dir_id})
     assert_contains(resp, swh_dir_id)
     assert_contains(resp, swh_dir_id_url)
 
     assert_contains(resp, "swh-take-new-snapshot")
 
+    _check_origin_view_title(resp, origin_info["url"], "directory")
+
 
 def _origin_branches_test_helper(client, origin_info, origin_snapshot):
-    url_args = {"origin_url": origin_info["url"]}
+    query_params = {"origin_url": origin_info["url"]}
 
-    url = reverse("browse-origin-branches", url_args=url_args)
+    url = reverse("browse-origin-branches", query_params=query_params)
 
     resp = client.get(url)
 
@@ -944,14 +1065,14 @@ def _origin_branches_test_helper(client, origin_info, origin_snapshot):
     origin_branches = origin_snapshot[0]
     origin_releases = origin_snapshot[1]
 
-    origin_branches_url = reverse("browse-origin-branches", url_args=url_args)
+    origin_branches_url = reverse("browse-origin-branches", query_params=query_params)
 
     assert_contains(
         resp,
         '<a href="%s">Branches (%s)</a>' % (origin_branches_url, len(origin_branches)),
     )
 
-    origin_releases_url = reverse("browse-origin-releases", url_args=url_args)
+    origin_releases_url = reverse("browse-origin-releases", query_params=query_params)
 
     nb_releases = len(origin_releases)
     if nb_releases > 0:
@@ -964,23 +1085,24 @@ def _origin_branches_test_helper(client, origin_info, origin_snapshot):
     for branch in origin_branches:
         browse_branch_url = reverse(
             "browse-origin-directory",
-            url_args={"origin_url": origin_info["url"]},
-            query_params={"branch": branch["name"]},
+            query_params={"origin_url": origin_info["url"], "branch": branch["name"]},
         )
         assert_contains(resp, '<a href="%s">' % escape(browse_branch_url))
 
         browse_revision_url = reverse(
             "browse-revision",
             url_args={"sha1_git": branch["revision"]},
-            query_params={"origin": origin_info["url"]},
+            query_params={"origin_url": origin_info["url"]},
         )
         assert_contains(resp, '<a href="%s">' % escape(browse_revision_url))
 
+    _check_origin_view_title(resp, origin_info["url"], "branches")
+
 
 def _origin_releases_test_helper(client, origin_info, origin_snapshot):
-    url_args = {"origin_url": origin_info["url"]}
+    query_params = {"origin_url": origin_info["url"]}
 
-    url = reverse("browse-origin-releases", url_args=url_args)
+    url = reverse("browse-origin-releases", query_params=query_params)
 
     resp = client.get(url)
     assert resp.status_code == 200
@@ -989,14 +1111,14 @@ def _origin_releases_test_helper(client, origin_info, origin_snapshot):
     origin_branches = origin_snapshot[0]
     origin_releases = origin_snapshot[1]
 
-    origin_branches_url = reverse("browse-origin-branches", url_args=url_args)
+    origin_branches_url = reverse("browse-origin-branches", query_params=query_params)
 
     assert_contains(
         resp,
         '<a href="%s">Branches (%s)</a>' % (origin_branches_url, len(origin_branches)),
     )
 
-    origin_releases_url = reverse("browse-origin-releases", url_args=url_args)
+    origin_releases_url = reverse("browse-origin-releases", query_params=query_params)
 
     nb_releases = len(origin_releases)
     if nb_releases > 0:
@@ -1010,17 +1132,19 @@ def _origin_releases_test_helper(client, origin_info, origin_snapshot):
         browse_release_url = reverse(
             "browse-release",
             url_args={"sha1_git": release["id"]},
-            query_params={"origin": origin_info["url"]},
+            query_params={"origin_url": origin_info["url"]},
         )
         browse_revision_url = reverse(
             "browse-revision",
             url_args={"sha1_git": release["target"]},
-            query_params={"origin": origin_info["url"]},
+            query_params={"origin_url": origin_info["url"]},
         )
 
         assert_contains(resp, '<a href="%s">' % escape(browse_release_url))
         assert_contains(resp, '<a href="%s">' % escape(browse_revision_url))
 
+    _check_origin_view_title(resp, origin_info["url"], "releases")
+
 
 @given(
     new_origin(), visit_dates(), revisions(min_size=10, max_size=10), existing_release()
@@ -1056,8 +1180,29 @@ def test_origin_branches_pagination_with_alias(
         new_origin["url"], visit.visit, status="full", snapshot=snp_dict["id"]
     )
 
-    url = reverse("browse-origin-branches", url_args={"origin_url": new_origin["url"]})
+    url = reverse(
+        "browse-origin-branches", query_params={"origin_url": new_origin["url"]}
+    )
     resp = client.get(url)
     assert resp.status_code == 200
     assert_template_used(resp, "browse/branches.html")
     assert_contains(resp, '<ul class="pagination')
+
+
+def _check_origin_view_title(resp, origin_url, object_type):
+    browse_origin_url = reverse(
+        "browse-origin", query_params={"origin_url": origin_url}
+    )
+
+    assert_contains(
+        resp,
+        textwrap.indent(
+            (
+                f"Browse archived {object_type} for origin\n"
+                f'<a href="{browse_origin_url}">\n'
+                f"  {origin_url}\n"
+                f"</a>"
+            ),
+            " " * 6,
+        ),
+    )
